@@ -2,17 +2,21 @@ lsmeans = function(object, specs, adjust=c("auto","tukey","sidak",p.adjust.metho
                    at, contr=list(), 
                    cov.reduce = function(x, name) mean(x), 
                    fac.reduce = function(coefs, lev) apply(coefs, 2, mean), 
-                   glhargs=NULL, 
-                   ##### NOW DEPRECATED #####  check.cells = TRUE, 
+                   glhargs=NULL, lf = FALSE,
                    ...) 
 {
     
     if (missing(specs)) stop("Must specify specs, e.g. 'pairwise ~ treatment'")
-    if(!is.null(glhargs)) # we'll pass contrasts to glht, if multcomp installed; else don't, and warn
+    if(!is.null(glhargs)) { # we'll pass contrasts to glht, if multcomp installed; else don't, and warn
         if(!require("multcomp")) {
             glhargs = NULL
             warning("'glhargs' option disabled because 'multcomp' package not installed")
         }
+        # force integer df since mvtnorm no longer supports fractional
+        else {
+            if (!is.null(glhargs$df)) glhargs$df = as.integer(max(1, floor(glhargs$df)))
+        }
+    }
     
     # for later use
     adjtbl = c("auto","tukey","sidak",p.adjust.methods)
@@ -242,8 +246,8 @@ lsmeans = function(object, specs, adjust=c("auto","tukey","sidak",p.adjust.metho
             if (nmat == 0) stop(paste("Can't predict at level", lev, "of", "facs.lbl"))
             else fac.reduce(X[matches, , drop=FALSE], lev)
         })
-        dimnames(K)[[2]] = apply(combs, 1, paste, collapse=", ")
-
+        rnames = dimnames(K)[[2]] = apply(combs, 1, paste, collapse=", ")
+        
     #### Here is the fcn I'll call to table an estimate of k'beta
         do.est = function(k) {
             est = se = df = NA
@@ -274,28 +278,32 @@ lsmeans = function(object, specs, adjust=c("auto","tukey","sidak",p.adjust.metho
         }
         
         # LS means
-        lsms = as.data.frame(t(apply(K,2,do.est)))
-        rnames = row.names(lsms)
-        # fix-up names and get CIs
-        names(lsms)[1] = "lsmean"
-        # include factor levels
-        lsms = cbind(combs, lsms)
-        if (conf > 1) conf = conf/100 # pct --> frac
-        if ((conf < 1) && (conf > .01)) {
-            if (is.null(ddfm)) {
-                me = qnorm((1-conf)/2, lower.tail=FALSE) * lsms$SE
-                lsms$asymp.LCL = lsms$lsmean - me
-                lsms$asymp.UCL = lsms$lsmean + me
-            }
-            else {
-                me = qt((1-conf)/2, lsms$df, lower.tail=FALSE) * lsms$SE
-                lsms$lower.CL = lsms$lsmean - me
-                lsms$upper.CL = lsms$lsmean + me
-            }
+        if (lf) {
+            results[[paste(facs.lbl, "lsmeans")]] = t(K)
         }
-        attr(lsms, "print.row.names") = FALSE
-        class(lsms) = c("data.frame.lsm", "data.frame")
-        results[[paste(facs.lbl, "lsmeans")]] = lsms
+        else {
+            lsms = as.data.frame(t(apply(K,2,do.est)))
+            # fix-up names and get CIs
+            names(lsms)[1] = "lsmean"
+            # include factor levels
+            lsms = cbind(combs, lsms)
+            if (conf > 1) conf = conf/100 # pct --> frac
+            if ((conf < 1) && (conf > .01)) {
+                if (is.null(ddfm)) {
+                    me = qnorm((1-conf)/2, lower.tail=FALSE) * lsms$SE
+                    lsms$asymp.LCL = lsms$lsmean - me
+                    lsms$asymp.UCL = lsms$lsmean + me
+                }
+                else {
+                    me = qt((1-conf)/2, lsms$df, lower.tail=FALSE) * lsms$SE
+                    lsms$lower.CL = lsms$lsmean - me
+                    lsms$upper.CL = lsms$lsmean + me
+                }
+            }
+            attr(lsms, "print.row.names") = FALSE
+            class(lsms) = c("data.frame.lsm", "data.frame")
+            results[[paste(facs.lbl, "lsmeans")]] = lsms
+        }        
         
         # Do requested contrasts
         if (! is.null(method)) {
@@ -324,7 +332,7 @@ lsmeans = function(object, specs, adjust=c("auto","tukey","sidak",p.adjust.metho
             
             # list to hold results
             Clist = list()
-            zer = rep(0, nrow(lsms))
+            zer = rep(0, ncol(K)) #### replaced nrow(lsms)) when lf arg added 
             
             # OK, let's go thru the bylist
             nby = length(bylist)
@@ -352,15 +360,21 @@ lsmeans = function(object, specs, adjust=c("auto","tukey","sidak",p.adjust.metho
                 #glhargs = NULL
                 warning("Error may occur in 'glht' due to rank deficiency")
             }
-            if (!is.null(glhargs)) { # for shipping it to glht
+            if (lf || !is.null(glhargs)) { # create linear fcn for glht
                 KK = t(sapply(Clist, function(con) {
                     nz = which(abs(con) > .0001)
                     K[ , nz] %*% con[nz]    
                 }))
-                # If glht gets fixed for rank deficiency, may want to consider checking rows of KK
-                # for estimability (see code in do.est())
-                args = c(list(model=object, linfct=KK[ , used]), glhargs)
-                ctbl = summary(do.call("glht", args))
+                if (lf) {
+                    dimnames(KK)[[2]] = row.names(K)
+                    ctbl = KK
+                }
+                else {
+                    # If glht gets fixed for rank deficiency, may want to consider checking rows of KK
+                    # for estimability (see code in do.est())
+                    args = c(list(model=object, linfct=KK[ , used]), glhargs)
+                    ctbl = summary(do.call("glht", args))
+                }
             }
             else { # internal way of doing contrasts
                 if (is.null(methdesc)) methdesc = method
@@ -398,31 +412,13 @@ lsmeans = function(object, specs, adjust=c("auto","tukey","sidak",p.adjust.metho
             results[[paste(facs.lbl,methdesc)]] = ctbl
         }
     }
-    class(results) = c("lsm","list")
+    if (!lf)
+        class(results) = c("lsm","list")
     results
 }
 
 
-### S3 print method for "lsm" class
-### We just print if data.frame, but summarize if glht class, because its print method
-### doesn't display the contrast estimates
-print.lsm = function(x, omit=NULL, ...) {
-    for (i in 1:length(x)) {
-        if (length(grep(i,omit)) > 0) next
-        cat(paste("$`", names(x)[i], "`\n", sep="")) # mimic print method for lists
-        elt = x[[i]]
-        if(inherits(elt, "glht")) print(summary(elt))
-        else {
-            print(elt)
-            msg = attr(elt, "mesg")
-            if (!is.null(msg)) 
-                for (j in 1:length(msg)) cat(paste("   ", msg[j], "\n"))
-            cat("\n")
-        }
-    }
-}
-
-# new version
+### S3 print method for "lsm" class - only reason we need this now is to support the 'omit' arg
 print.lsm = function(x, omit=NULL, ...) {
     for (i in 1:length(x)) {
         if (i %in% omit) next
@@ -433,6 +429,7 @@ print.lsm = function(x, omit=NULL, ...) {
     invisible(x)
 }
 
+### S3 print method for annotated data.frames
 print.data.frame.lsm = function(x, ...) {
     print.data.frame(x, row.names=attr(x, "print.row.names"))
     msg = attr(x, "mesg")
@@ -441,110 +438,3 @@ print.data.frame.lsm = function(x, ...) {
     invisible(x)
 }
 
-### functions to implement different families of contrasts
-### All return a matrix or data frame whose columns are the desired contrasts coefs
-### with appropriate row and column names
-### Also they have two attributes: 
-###   "desc" is an expanded description of the family,
-###   "adjust" is the default multiplicity adjustment (used if adjust="auto" in lsmeans)
-
-# all pairwise trt[i] - trt[j], i < j
-pairwise.lsmc = function(levs,...) {
-    k = length(levs)
-    M = data.frame(levs=levs)
-    for (i in 1:(k-1)) {
-        for (j in (i+1):k) {
-            con = rep(0,k)
-            con[i] = 1
-            con[j] = -1
-            nm = paste(levs[i], levs[j], sep = " - ")
-            M[[nm]] = con
-        }
-    }
-    row.names(M) = levs
-    M = M[-1]
-    attr(M, "desc") = "pairwise differences"
-    attr(M, "adjust") = "tukey"
-    M
-}
-
-# all pairwise trt[j] - trt[i], j > i
-revpairwise.lsmc = function(levs,...) {
-    k = length(levs)
-    M = data.frame(levs=levs)
-    for (i in 2:k) {
-        for (j in 1:(i-1)) {
-            con = rep(0,k)
-            con[i] = 1
-            con[j] = -1
-            nm = paste(levs[i], levs[j], sep = " - ")
-            M[[nm]] = con
-        }
-    }
-    row.names(M) = levs
-    M = M[-1]
-    attr(M, "desc") = "pairwise differences"
-    attr(M, "adjust") = "tukey"
-    M
-}
-
-# Poly contrasts - scaled w/ integer levels like most tables
-# ad hoc scaling works for up to 13 levels
-poly.lsmc = function(levs, max.degree=min(6,k-1)) {
-    nm = c("linear", "quadratic", "cubic", "quartic", paste("degree",5:20))
-    k = length(levs)
-    M = as.data.frame(poly(1:k, min(20,max.degree)))
-    for (j in 1:ncol(M)) {
-        con = M[ ,j]
-        pos = which(con > .01)
-        con = con / min(con[pos])
-        z = max(abs(con - round(con)))
-        while (z > .05) {
-            con = con / z
-            z = max(abs(con - round(con)))
-        }
-        M[ ,j] = round(con)
-    }
-    row.names(M) = levs
-    names(M) = nm[1:ncol(M)]
-    attr(M, "desc") = "polynomial contrasts"
-    attr(M, "adjust") = "none"
-    M
-}
-
-# All comparisons with a control; ref = index of control group
-# New version -- allows more than one control group (ref is a vector)
-trt.vs.ctrl.lsmc = function(levs, ref=1) {
-    if ((min(ref) < 1) || (max(ref) > length(levs)))
-        stop("Reference levels are out of range")
-    k = length(levs)
-    cnm = ifelse(length(ref)==1, 
-        levs[ref], 
-        paste("avg(", paste(levs[ref], collapse=","), ")", sep=""))
-    templ = rep(0, length(levs))
-    templ[ref] = -1 / length(ref)
-    M = data.frame(levs=levs)
-    for (i in 1:k) {
-        if (i %in% ref) next
-        con = templ
-        con[i] = 1
-        nm = paste(levs[i], cnm, sep = " - ")
-        M[[nm]] = con
-    }
-    row.names(M) = levs
-    M = M[-1]
-    attr(M, "desc") = "differences from control"
-    attr(M, "adjust") = "sidak"
-    M
-}
-
-# control is 1st level
-trt.vs.ctrl1.lsmc = function(levs, ...) {
-    trt.vs.ctrl.lsmc(levs, ref=1)
-}
-
-# control is last level
-trt.vs.ctrlk.lsmc = function(levs, ...) {
-    trt.vs.ctrl.lsmc(levs, ref=length(levs))
-}
-
